@@ -1,23 +1,22 @@
 """Tornado handlers for nbconvert."""
-
 # Copyright (c) Jupyter Development Team.
 # Distributed under the terms of the Modified BSD License.
-
 import io
 import os
 import zipfile
 
-from tornado import web, escape
+from anyio.to_thread import run_sync
+from ipython_genutils import text
+from ipython_genutils.py3compat import cast_bytes
+from nbformat import from_dict
+from tornado import web
 from tornado.log import app_log
 
-from ..base.handlers import (
-    JupyterHandler, FilesRedirectHandler,
-    path_regex,
-)
-from nbformat import from_dict
+from ..base.handlers import FilesRedirectHandler
+from ..base.handlers import JupyterHandler
+from ..base.handlers import path_regex
+from jupyter_server.utils import ensure_async
 
-from ipython_genutils.py3compat import cast_bytes
-from ipython_genutils import text
 
 def find_resource_files(output_files_dir):
     files = []
@@ -25,6 +24,7 @@ def find_resource_files(output_files_dir):
         files.extend([os.path.join(dirpath, f) for f in filenames])
     return files
 
+
 def respond_zip(handler, name, output, resources):
     """Zip up the output and resource files and respond with the zip file.
 
@@ -32,21 +32,21 @@ def respond_zip(handler, name, output, resources):
     files, in which case we serve the plain output file.
     """
     # Check if we have resource files we need to zip
-    output_files = resources.get('outputs', None)
+    output_files = resources.get("outputs", None)
     if not output_files:
         return False
 
     # Headers
-    zip_filename = os.path.splitext(name)[0] + '.zip'
+    zip_filename = os.path.splitext(name)[0] + ".zip"
     handler.set_attachment_header(zip_filename)
-    handler.set_header('Content-Type', 'application/zip')
-    handler.set_header('Cache-Control', 'no-store, no-cache, must-revalidate, max-age=0')
+    handler.set_header("Content-Type", "application/zip")
+    handler.set_header("Cache-Control", "no-store, no-cache, must-revalidate, max-age=0")
 
     # Prepare the zip file
     buffer = io.BytesIO()
-    zipf = zipfile.ZipFile(buffer, mode='w', compression=zipfile.ZIP_DEFLATED)
-    output_filename = os.path.splitext(name)[0] + resources['output_extension']
-    zipf.writestr(output_filename, cast_bytes(output, 'utf-8'))
+    zipf = zipfile.ZipFile(buffer, mode="w", compression=zipfile.ZIP_DEFLATED)
+    output_filename = os.path.splitext(name)[0] + resources["output_extension"]
+    zipf.writestr(output_filename, cast_bytes(output, "utf-8"))
     for filename, data in output_files.items():
         zipf.writestr(os.path.basename(filename), data)
     zipf.close()
@@ -54,6 +54,7 @@ def respond_zip(handler, name, output, resources):
     handler.finish(buffer.getvalue())
     return True
 
+
 def get_exporter(format, **kwargs):
     """get an exporter, raising appropriate errors"""
     # if this fails, will raise 500
@@ -77,51 +78,48 @@ def get_exporter(format, **kwargs):
 
 class NbconvertFileHandler(JupyterHandler):
 
-    SUPPORTED_METHODS = ('GET',)
+    SUPPORTED_METHODS = ("GET",)
 
     @web.authenticated
-    def get(self, format, path):
-
+    async def get(self, format, path):
+        self.check_xsrf_cookie()
         exporter = get_exporter(format, config=self.config, log=self.log)
 
-        path = path.strip('/')
+        path = path.strip("/")
         # If the notebook relates to a real file (default contents manager),
         # give its path to nbconvert.
-        if hasattr(self.contents_manager, '_get_os_path'):
+        if hasattr(self.contents_manager, "_get_os_path"):
             os_path = self.contents_manager._get_os_path(path)
             ext_resources_dir, basename = os.path.split(os_path)
         else:
             ext_resources_dir = None
 
-        model = self.contents_manager.get(path=path)
-        name = model['name']
-        if model['type'] != 'notebook':
+        model = await ensure_async(self.contents_manager.get(path=path))
+        name = model["name"]
+        if model["type"] != "notebook":
             # not a notebook, redirect to files
             return FilesRedirectHandler.redirect_to_files(self, path)
 
-        nb = model['content']
+        nb = model["content"]
 
-        self.set_header('Last-Modified', model['last_modified'])
+        self.set_header("Last-Modified", model["last_modified"])
 
         # create resources dictionary
-        mod_date = model['last_modified'].strftime(text.date_format)
+        mod_date = model["last_modified"].strftime(text.date_format)
         nb_title = os.path.splitext(name)[0]
 
         resource_dict = {
-            "metadata": {
-                "name": nb_title,
-                "modified_date": mod_date
-            },
-            "config_dir": self.application.settings['config_dir']
+            "metadata": {"name": nb_title, "modified_date": mod_date},
+            "config_dir": self.application.settings["config_dir"],
         }
 
         if ext_resources_dir:
-            resource_dict['metadata']['path'] = ext_resources_dir
+            resource_dict["metadata"]["path"] = ext_resources_dir
 
+        # Exporting can take a while, delegate to a thread so we don't block the event loop
         try:
-            output, resources = exporter.from_notebook_node(
-                nb,
-                resources=resource_dict
+            output, resources = await run_sync(
+                exporter.from_notebook_node(nb, resources=resource_dict)
             )
         except Exception as e:
             self.log.exception("nbconvert failed: %s", e)
@@ -131,36 +129,40 @@ class NbconvertFileHandler(JupyterHandler):
             return
 
         # Force download if requested
-        if self.get_argument('download', 'false').lower() == 'true':
-            filename = os.path.splitext(name)[0] + resources['output_extension']
+        if self.get_argument("download", "false").lower() == "true":
+            filename = os.path.splitext(name)[0] + resources["output_extension"]
             self.set_attachment_header(filename)
 
         # MIME type
         if exporter.output_mimetype:
-            self.set_header('Content-Type',
-                            '%s; charset=utf-8' % exporter.output_mimetype)
+            self.set_header("Content-Type", "%s; charset=utf-8" % exporter.output_mimetype)
 
-        self.set_header('Cache-Control', 'no-store, no-cache, must-revalidate, max-age=0')
+        self.set_header("Cache-Control", "no-store, no-cache, must-revalidate, max-age=0")
         self.finish(output)
 
 
 class NbconvertPostHandler(JupyterHandler):
 
-    SUPPORTED_METHODS = ('POST',)
+    SUPPORTED_METHODS = ("POST",)
 
     @web.authenticated
     def post(self, format):
         exporter = get_exporter(format, config=self.config)
 
         model = self.get_json_body()
-        name = model.get('name', 'notebook.ipynb')
-        nbnode = from_dict(model['content'])
+        name = model.get("name", "notebook.ipynb")
+        nbnode = from_dict(model["content"])
 
         try:
-            output, resources = exporter.from_notebook_node(nbnode, resources={
-                "metadata": {"name": name[:name.rfind('.')],},
-                "config_dir": self.application.settings['config_dir'],
-            })
+            output, resources = exporter.from_notebook_node(
+                nbnode,
+                resources={
+                    "metadata": {
+                        "name": name[: name.rfind(".")],
+                    },
+                    "config_dir": self.application.settings["config_dir"],
+                },
+            )
         except Exception as e:
             raise web.HTTPError(500, "nbconvert failed: %s" % e) from e
 
@@ -169,21 +171,19 @@ class NbconvertPostHandler(JupyterHandler):
 
         # MIME type
         if exporter.output_mimetype:
-            self.set_header('Content-Type',
-                            '%s; charset=utf-8' % exporter.output_mimetype)
+            self.set_header("Content-Type", "%s; charset=utf-8" % exporter.output_mimetype)
 
         self.finish(output)
 
 
-#-----------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
 # URL to handler mappings
-#-----------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
 
 _format_regex = r"(?P<format>\w+)"
 
 
 default_handlers = [
     (r"/nbconvert/%s" % _format_regex, NbconvertPostHandler),
-    (r"/nbconvert/%s%s" % (_format_regex, path_regex),
-         NbconvertFileHandler),
+    (r"/nbconvert/%s%s" % (_format_regex, path_regex), NbconvertFileHandler),
 ]
